/*
 * cmap.c
 *
 */

#include <stdio.h>
#include <xil/xil.h>
#include <X11/X.h>
#include <X11/Xlib.h>
#include <X11/Xatom.h>
#include <X11/Xutil.h>

#define MAXCOLORS 256

int
set_grayscale_cmap (Display *xdpy, Window xid, int ncolors)
{

    XVisualInfo	vinfo;
    Colormap 	cmap;
    unsigned long  pixels[MAXCOLORS], pmask[1], pixel[1];
    int i, j, k, index;
    XColor 	colors[MAXCOLORS];
    int		base;

    base = MAXCOLORS - ncolors;

    /* Find PseudoColor Visual */
    if((XMatchVisualInfo(xdpy,DefaultScreen(xdpy),8,PseudoColor,&vinfo)) == 0) {
    	fprintf(stderr,"Could not get 8 bit visual\n");
    	return (0);
    }

    /* Create X color map */
    cmap = XCreateColormap(xdpy, RootWindow(xdpy, xdpy->default_screen),
	vinfo.visual, AllocNone);
    i = XAllocColorCells(xdpy, cmap, 1, pmask, 0, pixels, 256);

    for (pixel[0] = 0; pixel[0] < base; pixel[0]++) {
    	XFreeColors(xdpy, cmap, pixel, 1, 0);
    }
    
    /* fill the colormap */
    index = 0;
    for (i = 0; i <ncolors; i++) {
    	colors[index].pixel = index + base ;
    	colors[index].blue = colors[index].green = colors[index].red =
	    (255 * i/(ncolors-1)) * 256;
    	colors[index].flags = DoRed | DoGreen | DoBlue;
    	index++;
    }


    XStoreColors(xdpy, cmap, colors, ncolors);

    XSetWindowColormap(xdpy, xid, cmap);
    XInstallColormap(xdpy,cmap);
    XFlush (xdpy);
    return ncolors;
}

void 
set_cmap (Display *xdpy, Window xid, XilLookup cmap)
{
int j, step = xil_lookup_get_output_nbands(cmap);

    XVisualInfo	vinfo;
    Colormap 	xcmap;
    unsigned long pixels[256], pmask[1], pixel[1];
    int		ncolors, base, index, i;
    Xil_unsigned8 cmap_data[MAXCOLORS*4];
    XColor 	colors[MAXCOLORS];

    ncolors = xil_lookup_get_num_entries (cmap);
    base = xil_lookup_get_offset (cmap);
    xil_lookup_get_values (cmap, base, ncolors, cmap_data);

    /* Find PseudoColor Visual */
    if((XMatchVisualInfo(xdpy,DefaultScreen(xdpy),8,PseudoColor,&vinfo)) 
       == 0) {
       fprintf(stderr,"XMatchVisualInfo failed; could not get 8 bit visual\n");
       return;
    }

    /* Create X color map */
    xcmap = XCreateColormap(xdpy, RootWindow(xdpy, xdpy->default_screen),
	vinfo.visual, AllocNone);
    if (XAllocColorCells(xdpy, xcmap, 1, pmask, 0, pixels, base + ncolors)
	== 0) {
       fprintf(stderr,"XAllocColorCells failed\n");
       return;
    }

    for (pixel[0] = 0; pixel[0] < base; pixel[0]++) {
    	XFreeColors(xdpy, xcmap, pixel, 1, 0);
    }

    /* fill the colormap */
#ifdef NOTDEF
    for (i = index = 0; index < ncolors; index++) {
       colors[index].pixel = base + index;
       colors[index].blue  = cmap_data[i++]<<8;
       colors[index].green = cmap_data[i++]<<8;
       colors[index].red   = cmap_data[i++]<<8;
       colors[index].flags = DoRed | DoGreen | DoBlue;
    }
#else
    for(i=0,j=step-3; i<ncolors; i++,j+=step) {
        colors[i].pixel  = base+i;
        colors[i].flags  = DoRed | DoGreen | DoBlue;
        colors[i].blue   = cmap_data[j]<<8;
        colors[i].green  = cmap_data[j+1]<<8;
        colors[i].red    = cmap_data[j+2]<<8;
    }
#endif
    
    XStoreColors(xdpy, xcmap, colors, ncolors);

    XSetWindowColormap (xdpy, xid, xcmap);
    XInstallColormap (xdpy, xcmap);
    XFlush (xdpy);
}
