/*
   This example reads an image from a file
   and displays the image in a Xwindow
*/

#include <stdio.h>
#include <stdlib.h>
#include <xil/xil.h>

#define	NCOLORS		216
#define THRESHOLD       100

XilImage 
load_file (XilSystemState state, char* filename)
{
   XilImage image;
   int width, height, nbands, bits;
   XilDataType datatype;
   XilMemoryStorage* storage;
   FILE *in_file;
   char datafile[256];
   int size;
	   
   /* open the header file */
   if ((in_file = fopen(filename, "r")) == NULL) {
      fprintf (stderr,"could not open %s\n", filename);
      exit(1);
   }
	   
   /* get header information */
   fscanf(in_file, "%d%d%d%d", &width, &height, &bits, &nbands);
   fscanf(in_file, "%s", datafile);
   fclose(in_file);
	   
   switch (bits) {
    case 1:
      datatype = XIL_BIT;
      break;
    case 8:
      datatype = XIL_BYTE;
      break;
    case 16:
	datatype = XIL_SHORT;
	break;
    default:
      fprintf(stderr,"this program doesn't support %d bit images\n",bits);
      exit(1);
   }
	   
   /* open the data file */
   if ((in_file = fopen(datafile, "r")) == NULL) {
      fprintf (stderr,"could not open %s\n", datafile);
      exit(1);
   }
	   
   /* create the image to read the data into */
   image = xil_create (state, width, height, nbands, datatype);
   if (!image) {
      fprintf (stderr,"image create failed\n");
      exit(1);
   }

   /* get the pointer to the image data */
   xil_export (image);
   xil_get_memory_storage(image, storage);
   if (storage == NULL) {
      fprintf(stderr, "could not get image storage info\n");
      exit(1);
   }

   /* load the image data */
   switch (datatype) {
    case XIL_BIT:
      size = nbands * ((width + 7) / 8) * height;
      if (fread((char *) (*storage).bit.data, sizeof(Xil_unsigned8), 
		size, in_file) != size){
	 fprintf (stderr, "error reading data in %s\n", datafile);
	 exit(1);
      }
      break;
    case XIL_BYTE:
      size = nbands * width * height;
      if (fread((char *) (*storage).byte.data, sizeof(Xil_unsigned8), 
		size, in_file) != size) {
	 fprintf (stderr, "error reading data in %s\n", datafile);
	 exit(1);
      }
      break;
    case XIL_SHORT:
      size = nbands * width * height;
      if (fread((char *) (*storage).shrt.data, sizeof(Xil_signed16), 
		size, in_file) != size) {
	 fprintf (stderr, "error reading data in %s\n", datafile);
	 exit(1);
      }
      break;
   }
   fclose(in_file);
   
   /* make the image available for xil processing */
   xil_import (image, FALSE);
   return (image);
}


main (int argc, char ** argv)
{
   char filename[256];
   XilSystemState state;
   XilImage image, bit_image, retained_image, display;
   unsigned int width, height, nbands;
   XilDataType datatype;
   Display *xdisplay;
   Window xwindow;
   XEvent event;
   int display_depth;
   float mult[1], offset[1];
   float lowvalue[1], highvalue[1], mapvalue[1];

	     XilLookup cmap;
	     XilDitherMask mask;
	     XilImage capture_image;
	     int multipliers[3];
	     unsigned int dimensions[3];
	     float scale_x, scale_y;

	     float dmask[] = {
		0.0, 0.5, 0.125, 0.625,
		0.75, 0.25, 0.875, 0.375,
		0.1875, 0.6875, 0.125, 0.5625,
		0.9375, 0.4275, 0.8125, 0.3125,

		0.0, 0.5, 0.125, 0.625,
		0.75, 0.25, 0.875, 0.375,
		0.1875, 0.6875, 0.125, 0.5625,
		0.9375, 0.4275, 0.8125, 0.3125,

		0.0, 0.5, 0.125, 0.625,
		0.75, 0.25, 0.875, 0.375,
		0.1875, 0.6875, 0.125, 0.5625,
		0.9375, 0.4275, 0.8125, 0.3125 }; 

	     dimensions[0] = 4;
	     dimensions[1] = 9;
	     dimensions[2] = 6;
	     multipliers[0] = 1;
	     multipliers[1] = 4;
	     multipliers[2] = 36;

   /* get filename from command line */
   *filename = NULL;
   while (--argc){
      if ((*++argv)[0] == '-')
	 switch ((*argv)[1]){
	 case 'h':
	    fprintf (stderr, "usage: display [header filename]\n");
	    break;
	 }
      else {
	 strcpy (filename, *argv);
      }
   }
   state = xil_open();
   if (state == NULL) {
      fprintf (stderr, "unable to open xil library\n");
      exit(1);
   }
/*
   xil_state_set_synchronize(state,TRUE);
*/

   if (strcmp(filename, "vfc"))
   {
	if (*filename == NULL)
	{
	    fprintf (stderr, "no filename specified, attempting to open vfc device...");
	    if (!(image = xil_create_from_device(state , "SUNWvfc", (void *)0)))
	    {
		fprintf(stderr, "failed to open vfc device\n");
		xil_close(state);
		exit(1);
	    }
	}
	else
	{
	    image = load_file(state, filename);
	}
   }

   xil_get_info (image, &width, &height, &nbands, &datatype);

   /* xlib window creation */
   xdisplay = XOpenDisplay (NULL);
   if (!xdisplay) {
      fprintf (stderr, "Unable to connect to server\n");
      exit (1);
   }
   display_depth = DefaultDepth(xdisplay, DefaultScreen(xdisplay));
   xwindow = XCreateSimpleWindow (xdisplay, DefaultRootWindow(xdisplay),
				  20, 20, width, height, 0, 0, 0);
   if (!xwindow) {
      fprintf (stderr, "Unable to create window\n");
      exit (1);
   }
	   /* list window events used */
   XSelectInput (xdisplay, xwindow, ExposureMask | ButtonPressMask);
	   /* make the window visible */
   XMapWindow (xdisplay, xwindow);
      
    if ((display_depth != 8) || (nbands != 3) || (datatype != XIL_BYTE))
    {
	fprintf(stderr, "configuration incorrect, %d %d\n", display_depth, nbands);
	exit(-1);
    }
#define COLOR

#ifdef COLOR
		mask = xil_dithermask_create (state, 3, 3, nbands, dmask);
		cmap = xil_colorcube_create (state, datatype, datatype, nbands,
				  256-NCOLORS, multipliers, dimensions);
	set_cmap (xdisplay, xwindow, cmap);
#else GRAYSCALE
	set_grayscale_cmap(xdisplay, xwindow, 128);
#endif
	scale_x = 360.0/640;
	scale_y = 1.0/2;
#define NEW 1
#ifdef NEW
	capture_image = xil_create (state, width*scale_x, height*scale_y, 4, datatype);
	capture_image = xil_create_child (capture_image, 0,0,width*scale_x, height*scale_y, 1, 3);
	retained_image = xil_create (state, width*scale_x, height*scale_y, 1, datatype);
	xil_scale(image, capture_image, "nearest", scale_x, scale_y);
#ifdef COLOR
        xil_ordered_dither(capture_image, retained_image, cmap, mask);
#else GRAYSCALE
	xil_set_colorspace(retained_image, xil_colorspace_get_by_name(state, "grayscale"));
	xil_set_colorspace(capture_image,xil_colorspace_get_by_name(state,  "rgblinear"));
        xil_color_convert(capture_image, retained_image);
#endif
#else
	retained_image = xil_create (state, width, height, 1, datatype);
	xil_ordered_dither(image, retained_image, cmap, mask);
#endif
	   
   /* wait for the window to be mapped (an Expose event) */
   while (1) {
      XNextEvent (xdisplay, &event);
      if (event.xany.type == Expose) break; 
   }

   display = xil_create_from_window (state, xdisplay, xwindow);
   xil_copy (retained_image, display);
	   
   /*refresh window as necessary till program terminates */
   while (1)
   {
#ifdef OLD
      XNextEvent (xdisplay, &event);
      if (event.xany.type == Expose) 
      {
	xil_scale(image, capture_image, "nearest", scale_x, scale_y);
	xil_ordered_dither(capture_image, retained_image, cmap, mask);
	xil_copy (retained_image, display);
      }
      else if (event.xany.type == ButtonPress)
	break;
      else
	fprintf(stderr, "event %d\n", event.xany.type);
#else
	xil_scale(image, capture_image, "nearest", scale_x, scale_y);
#ifdef COLOR
	xil_ordered_dither(capture_image, display, cmap, mask);
#else GRAYSCALE
	xil_color_convert(capture_image, display);
#endif
	xil_toss(capture_image);
#endif
   }
   /* cleanup */
   XCloseDisplay (xdisplay);
   xil_destroy (image);
   xil_close (state);
}
